////////////////////////////////////////////////////////////////////////
// $Id: DBSCANfinderAna.cxx,v 1.36 2010/09/15  bpage Exp $
//
// module to create a TTree for analysis
//
// \author tjyang@fnal.gov
//
////////////////////////////////////////////////////////////////////////
#ifndef ANALYSISTREE_H
#define ANALYSISTREE_H

// Framework includes
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Principal/Event.h" 
#include "art/Framework/Principal/SubRun.h" 
#include "fhiclcpp/ParameterSet.h" 
#include "art/Framework/Principal/Handle.h" 
#include "art/Framework/Principal/View.h" 
#include "canvas/Persistency/Common/Ptr.h" 
#include "canvas/Persistency/Common/PtrVector.h" 
#include "art/Framework/Services/Registry/ServiceHandle.h" 
#include "art/Framework/Services/Optional/TFileService.h" 
#include "art/Framework/Services/Optional/TFileDirectory.h" 
#include "messagefacility/MessageLogger/MessageLogger.h" 
#include "canvas/Persistency/Common/FindMany.h"

#include "T962_Objects/MINOS.h"
#include "T962_Objects/Paddles.h"

#include "larcore/Geometry/Geometry.h"
#include "nusimdata/SimulationBase/MCTruth.h"
#include "nusimdata/SimulationBase/MCFlux.h"
#include "lardataobj/Simulation/SimChannel.h"
#include "lardataobj/AnalysisBase/Calorimetry.h"
#include "lardataobj/AnalysisBase/ParticleID.h"
#include "lardataobj/RawData/RawDigit.h"
#include "lardataobj/RawData/BeamInfo.h"
#include "lardata/Utilities/AssociationUtil.h"
#include "lardata/DetectorInfoServices/DetectorPropertiesService.h"
#include "larcoreobj/SummaryData/POTSummary.h"
#include "larsim/MCCheater/BackTracker.h"
#include "lardataobj/RecoBase/Track.h"
#include "lardataobj/RecoBase/Cluster.h"
#include "lardataobj/RecoBase/Wire.h"
#include "lardataobj/RecoBase/Hit.h"
#include "lardataobj/RecoBase/EndPoint2D.h"
#include "lardataobj/RecoBase/Vertex.h"
#include "larcoreobj/SimpleTypesAndConstants/geo_types.h"

#include <vector>
#include <cmath> // std::tan()
#include <string>
#include <sstream>
#include <fstream>
#include <algorithm>
#include <functional>

#include "TTree.h"
#include "TFile.h"
#include "TH1.h"
#include "TSystem.h"
#include "TString.h"
#include "TTimeStamp.h"

const int kMaxTrack      = 1000;  //maximum number of tracks
const int kMaxClusters   = 1000;  //maximum number of clusters
const int kMaxHits       = 20000; //maximum number of hits;
const int kMaxPrimaries  = 20000;  //maximum number of primary particles
const int kMaxTrackHits  = 1000;  //maximum number of hits on a track

///Track finding and building 
namespace t962 {
   
  class AnalysisTreeT962 : public art::EDAnalyzer {

  public:
          
    explicit AnalysisTreeT962(fhicl::ParameterSet const& pset); 
    virtual ~AnalysisTreeT962();
 
    /// read access to event
    void analyze(const art::Event& evt);
    void beginJob();
    void beginSubRun(const art::SubRun& sr);
    
  protected:
  
    bool EndsOnBoundary(art::Ptr<recob::Track> lar_track);
  
  private:
    
    void HitsPurity(std::vector< art::Ptr<recob::Hit> > const& hits, int& trackid, double& purity, double& maxe);
    void ResetVars();

    TTree* fTree;
    //run information
    int run;
    int subrun;
    int event;
    double evttime;
    double beamtime;
    double pot;
    double taulife;
    int isdata;
    //reconstructed information    
    double vtxx_reco;
    double vtxy_reco;
    double vtxz_reco;
    int nvertices;
    double vtx_x[kMaxTrack];
    double vtx_y[kMaxTrack];
    double vtx_z[kMaxTrack];
    double vtx_w0[kMaxTrack];
    double vtx_w1[kMaxTrack];
    double vtx_t0[kMaxTrack];
    double vtx_t1[kMaxTrack];
    int nclusu_reco;
    int nclusv_reco;
    int nclusw_reco;
    int ntracks_reco;         //number of reconstructed tracks
    int nvertextracks_reco;   //number of reconstructed tracks with start position within fvertextrackWindow cm of vertex
    int ntrackendonboundary_reco; //number of reconstructed tracks with end poistion within fboundaryWindow cm of detector boundary
    int nvertexclustersu_reco; //number of reconstructed clusters with start position within fvertexclusterWindow cm of vertex
    int nvertexclustersv_reco; //number of reconstructed clusters with start position within fvertexclusterWindow cm of vertex
    int nvertexclustersw_reco; //number of reconstructed clusters with start position within fvertexclusterWindow cm of vertex
    double nearestx[3];
    double nearesty[3];
    double nearestz[3];
    double trackstart_x_reco;
    double trackstart_y_reco;
    double trackstart_z_reco;
    double trackexit_x_reco;
    double trackexit_y_reco;
    double trackexit_z_reco;
    double trackstart_dcosx_reco;
    double trackstart_dcosy_reco;
    double trackstart_dcosz_reco;       
    double trackexit_dcosx_reco;
    double trackexit_dcosy_reco;
    double trackexit_dcosz_reco;   
    double enu_reco;
    int    nclupertrack_reco;  //how many times one cluster is used in tracking?
    double trkvtxx[kMaxTrack];
    double trkvtxy[kMaxTrack];
    double trkvtxz[kMaxTrack];
    double trkendx[kMaxTrack];
    double trkendy[kMaxTrack];
    double trkendz[kMaxTrack];
    double trkstartdcosx[kMaxTrack];
    double trkstartdcosy[kMaxTrack];
    double trkstartdcosz[kMaxTrack];
    double trkenddcosx[kMaxTrack];
    double trkenddcosy[kMaxTrack];
    double trkenddcosz[kMaxTrack];
    double trkke[kMaxTrack][2];
    double Kin_Eng_reco[kMaxTrack][2];
    double trkrange[kMaxTrack][2];
    double trkpitchc[kMaxTrack][2];
    int    nmaxtrkhits[2];
    int    ntrkhits[kMaxTrack][2];
    double trkdedx[kMaxTrack][2][kMaxTrackHits];
    double trkdqdx[kMaxTrack][2][kMaxTrackHits];
    double trkresrg[kMaxTrack][2][kMaxTrackHits];
    int    trkhitindexc[kMaxTrack][2][kMaxTrackHits];
    int    trkpid[kMaxTrack][2];
    int    trkpidndf[kMaxTrack][2];
    double trkpidchi2[kMaxTrack][2];
    double trkpida[kMaxTrack][2];
    double trkmissinge[kMaxTrack][2];
    double trkmissingeavg[kMaxTrack][2];
    int    trktruepdgu[kMaxTrack];
    double trktrueeffu[kMaxTrack];
    double trktruepuru[kMaxTrack];
    int    trktruepdgv[kMaxTrack];
    double trktrueeffv[kMaxTrack];
    double trktruepurv[kMaxTrack];
    int    trkg4Id[kMaxTrack];

    //matching information
    int nmatched_reco;        //number of matched tracks
    double trk_mom_minos;
    double trk_charge_minos;
    double trk_dcosx_minos;
    double trk_dcosy_minos;
    double trk_dcosz_minos;
    double trk_vtxx_minos;
    double trk_vtxy_minos;
    double trk_vtxz_minos;
    int trk_index_minos;
    int trkcontained_minos;
    int test_charge_minos;
    double rdiff_minos;
    double xdiff_minos;
    double ydiff_minos;
    double thetadiff_minos;
    double muon_Kin_Eng_reco;
    int no_dead_wires_muon;
    
    //mctruth information
    int parpdg;
    double parmom;
    int nuPDG_truth;
    int ccnc_truth;
    int mode_truth;
    double enu_truth;
    double Q2_truth;
    double W_truth;
    double nuvtxx_truth;
    double nuvtxy_truth;
    double nuvtxz_truth;
    double nu_dcosx_truth;
    double nu_dcosy_truth;
    double nu_dcosz_truth;
    double lep_mom_truth;
    double lep_dcosx_truth;
    double lep_dcosy_truth;
    double lep_dcosz_truth;
    int    mcevts_truth;
    double beamwgt;
    double tpx_flux;
    double tpy_flux;
    double tpz_flux;
    int    tptype_flux;
    
    float  mc_index_minos;
    double mc_pdg_minos;
    double mc_px_minos;
    double mc_py_minos;
    double mc_pz_minos;
    double mc_ene_minos;
    double mc_mass_minos;
    double mc_vtxx_minos;
    double mc_vtxy_minos;
    double mc_vtxz_minos;
    int hitnuc_truth;

    int no_clusters;

    int clusters_planeNo[kMaxClusters];
    double Start_pt_w_Cl[kMaxClusters];
    double Start_pt_t_Cl[kMaxClusters];
    double End_pt_w_Cl[kMaxClusters];
    double End_pt_t_Cl[kMaxClusters];

    int fNhitsClust[kMaxClusters];

    double two_trackstart_dcosx_reco[2];
    double two_trackstart_dcosy_reco[2];
    double two_trackstart_dcosz_reco[2];
    double two_trackexit_dcosx_reco[2];
    double two_trackexit_dcosy_reco[2];
    double two_trackexit_dcosz_reco[2];

    int no_primaries;
    int geant_list_size;
    int pdg[kMaxPrimaries];
    double Eng[kMaxPrimaries];
    double Px[kMaxPrimaries];
    double Py[kMaxPrimaries];
    double Pz[kMaxPrimaries];
    double StartPointx[kMaxPrimaries];
    double StartPointy[kMaxPrimaries];
    double StartPointz[kMaxPrimaries];
    double EndPointx[kMaxPrimaries];
    double EndPointy[kMaxPrimaries];
    double EndPointz[kMaxPrimaries];
    int NumberDaughters[kMaxPrimaries];
    int TrackId[kMaxPrimaries];
    int Mother[kMaxPrimaries];
    int process_primary[kMaxPrimaries];
    int orig[kMaxPrimaries];

    //from genie:
    int genie_no_primaries;
    int genie_primaries_pdg[kMaxPrimaries];
    double genie_Eng[kMaxPrimaries];
    double genie_Px[kMaxPrimaries];
    double genie_Py[kMaxPrimaries];
    double genie_Pz[kMaxPrimaries];
    double genie_P[kMaxPrimaries];
    int genie_status_code[kMaxPrimaries];
    double genie_mass[kMaxPrimaries];
    int genie_trackID[kMaxPrimaries];
    int genie_ND[kMaxPrimaries];
    int genie_mother[kMaxPrimaries];
       
    int no_hits;
    int hit_plane[kMaxHits];
    int hit_wire[kMaxHits];
    int hit_channel[kMaxHits];
    double hit_peakT[kMaxHits];
    double hit_peakTcorr[kMaxHits];
    double hit_charge[kMaxHits];
    double hit_ph[kMaxHits];
    int    hit_trkid[kMaxHits];    
    double hit_etruth[kMaxHits];

    //paddle information
    double pmttime;
    int    pmt1[4];
    int    pmt2[4];
    int    pmt3[4];
    int    pmt4[4];

    std::string fDigitModuleLabel;
    std::string fHitsModuleLabel;
    std::string fLArG4ModuleLabel;
    std::string fCalDataModuleLabel; 
    std::string fGenieGenModuleLabel;
    std::string fG4ModuleLabel;
    std::string fClusterModuleLabel;   
    std::string fTrackModuleLabel;
    std::string fEndPoint2DModuleLabel;
    std::string fVertexModuleLabel;
    std::string fMINOSModuleLabel;
    std::string fTrackMatchModuleLabel;
    std::string fPOTModuleLabel;
    std::string fCalorimetryModuleLabel;
    std::string fParticleIDModuleLabel;
    std::string fPaddlesModuleLabel;

    double fvertextrackWindow;
    double fvertexclusterWindow;
    double fboundaryWindow;

    TH1D *hBeamWeight_numu_numode;

  }; // class AnalysisTreeT962
}

const double MPi  = 0.1396;
const double MPro = 0.938272;
const double MMu  = 0.1057;
double LifetimeCorrection(float time){

   double t = time;

   //auto const* LArProp = lar::providerFrom<detinfo::LArPropertiesService>();
   auto const* detprop = lar::providerFrom<detinfo::DetectorPropertiesService>();

   double timetick = detprop->SamplingRate()*1.e-3;    //time sample in microsec
   double presamplings = detprop->TriggerOffset();

   t -= presamplings;
   time = t * timetick;  //  (in microsec)

   // Note: LArProperties::ElectronLifetime() has moved to DetectorProperties/DetectorPropertiesService
   double tau = detprop->ElectronLifetime();

   double correction = exp(time/tau);
   return correction;
}


//-------------------------------------------------

t962::AnalysisTreeT962::AnalysisTreeT962(fhicl::ParameterSet const& pset) :
  EDAnalyzer(pset), 
  fDigitModuleLabel         (pset.get< std::string >("DigitModuleLabel")        ),
  fHitsModuleLabel          (pset.get< std::string >("HitsModuleLabel")         ),
  fLArG4ModuleLabel         (pset.get< std::string >("LArGeantModuleLabel")     ),
  fCalDataModuleLabel       (pset.get< std::string >("CalDataModuleLabel")      ),
  fGenieGenModuleLabel      (pset.get< std::string >("GenieGenModuleLabel")     ),
  fG4ModuleLabel            (pset.get< std::string >("G4ModuleLabel")           ),
  fClusterModuleLabel       (pset.get< std::string >("ClusterModuleLabel")      ),
  fTrackModuleLabel         (pset.get< std::string >("TrackModuleLabel")        ),
  fEndPoint2DModuleLabel    (pset.get< std::string >("EndPoint2DModuleLabel")   ),
  fVertexModuleLabel        (pset.get< std::string >("VertexModuleLabel")       ),
  fMINOSModuleLabel         (pset.get< std::string >("MINOSModuleLabel")        ),
  fTrackMatchModuleLabel    (pset.get< std::string >("TrackMatchModuleLabel")   ),
  fPOTModuleLabel           (pset.get< std::string >("POTModuleLabel")          ),
  fCalorimetryModuleLabel   (pset.get< std::string >("CalorimetryModuleLabel")  ),
  fParticleIDModuleLabel    (pset.get< std::string >("ParticleIDModuleLabel")   ),
  fPaddlesModuleLabel       (pset.get< std::string >("PaddlesModuleLabel")      ),
  fvertextrackWindow        (pset.get< double >("vertextrackWindow")            ),
  fvertexclusterWindow      (pset.get< double >("vertexclusterWindow")          ),
  fboundaryWindow           (pset.get< double >("boundaryWindow")               )
  
{
}

//-------------------------------------------------
t962::AnalysisTreeT962::~AnalysisTreeT962()
{
}

void t962::AnalysisTreeT962::beginJob()
{
  art::ServiceHandle<art::TFileService> tfs;
  fTree = tfs->make<TTree>("anatree","analysis tree");
  fTree->Branch("run",&run,"run/I");
  fTree->Branch("subrun",&subrun,"subrun/I");
  fTree->Branch("event",&event,"event/I");
  fTree->Branch("evttime",&evttime,"evttime/D");
  fTree->Branch("beamtime",&beamtime,"beamtime/D");
  fTree->Branch("pot",&pot,"pot/D");
  fTree->Branch("isdata",&isdata,"isdata/I");
  fTree->Branch("taulife",&taulife,"taulife/D");
  fTree->Branch("vtxx_reco",&vtxx_reco,"vtxx_reco/D");
  fTree->Branch("vtxy_reco",&vtxy_reco,"vtxy_reco/D");
  fTree->Branch("vtxz_reco",&vtxz_reco,"vtxz_reco/D");
  fTree->Branch("nclusu_reco",&nclusu_reco,"nclusu_reco/I");
  fTree->Branch("nclusv_reco",&nclusv_reco,"nclusv_reco/I");
  fTree->Branch("nclusw_reco",&nclusw_reco,"nclusw_reco/I");
  fTree->Branch("ntracks_reco",&ntracks_reco,"ntracks_reco/I");
  fTree->Branch("nvertextracks_reco",&nvertextracks_reco,"nvertextracks_reco/I");
  fTree->Branch("nvertexclustersu_reco",&nvertexclustersu_reco,"nvertexclustersu_reco/I");
  fTree->Branch("nvertexclustersv_reco",&nvertexclustersv_reco,"nvertexclustersv_reco/I");
  fTree->Branch("nvertexclustersw_reco",&nvertexclustersw_reco,"nvertexclustersw_reco/I");
  fTree->Branch("ntrackendonboundary_reco",&ntrackendonboundary_reco,"ntrackendonboundary_reco/I");
  fTree->Branch("nearestx",nearestx,"nearestx[3]/D");
  fTree->Branch("nearesty",nearesty,"nearesty[3]/D");
  fTree->Branch("nearestz",nearestz,"nearestz[3]/D");
  fTree->Branch("trackstart_dcosx_reco",&trackstart_dcosx_reco, "trackstart_dcosx_reco/D");
  fTree->Branch("trackstart_dcosy_reco",&trackstart_dcosy_reco, "trackstart_dcosy_reco/D");
  fTree->Branch("trackstart_dcosz_reco",&trackstart_dcosz_reco, "trackstart_dcosz_reco/D");
  fTree->Branch("trackexit_dcosx_reco",&trackexit_dcosx_reco, "trackexit_dcosx_reco/D");
  fTree->Branch("trackexit_dcosy_reco",&trackexit_dcosy_reco, "trackexit_dcosy_reco/D");
  fTree->Branch("trackexit_dcosz_reco",&trackexit_dcosz_reco, "trackexit_dcosz_reco/D");
  fTree->Branch("trackstart_x_reco",&trackstart_x_reco, "trackstart_x_reco/D");
  fTree->Branch("trackstart_y_reco",&trackstart_y_reco, "trackstart_y_reco/D");
  fTree->Branch("trackstart_z_reco",&trackstart_z_reco, "trackstart_z_reco/D");
  fTree->Branch("trackexit_x_reco",&trackexit_x_reco, "trackexit_x_reco/D");
  fTree->Branch("trackexit_y_reco",&trackexit_y_reco, "trackexit_y_reco/D");
  fTree->Branch("trackexit_z_reco",&trackexit_z_reco, "trackexit_z_reco/D");    
  fTree->Branch("enu_reco",&enu_reco,"enu_reco/D");
  fTree->Branch("nclupertrack_reco",&nclupertrack_reco,"nclupertrack_reco/I");
  fTree->Branch("trkvtxx",trkvtxx,"trkvtxx[ntracks_reco]/D");
  fTree->Branch("trkvtxy",trkvtxy,"trkvtxy[ntracks_reco]/D");
  fTree->Branch("trkvtxz",trkvtxz,"trkvtxz[ntracks_reco]/D");
  fTree->Branch("trkendx",trkendx,"trkendx[ntracks_reco]/D");
  fTree->Branch("trkendy",trkendy,"trkendy[ntracks_reco]/D");
  fTree->Branch("trkendz",trkendz,"trkendz[ntracks_reco]/D");
  fTree->Branch("trkstartdcosx",trkstartdcosx,"trkstartdcosx[ntracks_reco]/D");
  fTree->Branch("trkstartdcosy",trkstartdcosy,"trkstartdcosy[ntracks_reco]/D");
  fTree->Branch("trkstartdcosz",trkstartdcosz,"trkstartdcosz[ntracks_reco]/D");
  fTree->Branch("trkenddcosx",trkenddcosx,"trkenddcosx[ntracks_reco]/D");
  fTree->Branch("trkenddcosy",trkenddcosy,"trkenddcosy[ntracks_reco]/D");
  fTree->Branch("trkenddcosz",trkenddcosz,"trkenddcosz[ntracks_reco]/D");
  fTree->Branch("trkke",trkke,"trkke[ntracks_reco][2]/D");
  fTree->Branch("trkrange",trkrange,"trkrange[ntracks_reco][2]/D");
  fTree->Branch("trkpitchc",trkpitchc,"trkpitchc[ntracks_reco][2]/D");
  fTree->Branch("nmaxtrkhits",&nmaxtrkhits,"nmaxtrkhits[2]/I");
  fTree->Branch("ntrkhits",ntrkhits,"ntrkhits[ntracks_reco][2]/I");
  fTree->Branch("trkdedx",trkdedx,"trkdedx[ntracks_reco][2][1000]/D");
  fTree->Branch("trkdqdx",trkdqdx,"trkdqdx[ntracks_reco][2][1000]/D");
  fTree->Branch("trkresrg",trkresrg,"trkresrg[ntracks_reco][2][1000]/D");
  fTree->Branch("trkhitindexc",trkhitindexc,"trkhitindexc[ntracks_reco][2][1000]/I");
  fTree->Branch("trkpid",trkpid,"trkpid[ntracks_reco][2]/I");
  fTree->Branch("trkpidndf",trkpidndf,"trkpidndf[ntracks_reco][2]/I");
  fTree->Branch("trkpidchi2",trkpidchi2,"trkpidchi2[ntracks_reco][2]/D");
  fTree->Branch("trkpida",trkpida,"trkpida[ntracks_reco][2]/D");
  fTree->Branch("trkmissinge",trkmissinge,"trkmissinge[ntracks_reco][2]/D");
  fTree->Branch("trkmissingeavg",trkmissingeavg,"trkmissingeavg[ntracks_reco][2]/D");
  fTree->Branch("trktruepdgu",trktruepdgu,"trktruepdgu[ntracks_reco]/I");
  fTree->Branch("trktrueeffu",trktrueeffu,"trktrueeffu[ntracks_reco]/D");
  fTree->Branch("trktruepuru",trktruepuru,"trktruepuru[ntracks_reco]/D");
  fTree->Branch("trktruepdgv",trktruepdgv,"trktruepdgv[ntracks_reco]/I");
  fTree->Branch("trktrueeffv",trktrueeffv,"trktrueeffv[ntracks_reco]/D");
  fTree->Branch("trktruepurv",trktruepurv,"trktruepurv[ntracks_reco]/D");  
  fTree->Branch("trkg4Id",trkg4Id,"trkg4Id[ntracks_reco]/I");
  fTree->Branch("nmatched_reco",&nmatched_reco,"nmatched_reco/I");  
  fTree->Branch("trk_mom_minos",&trk_mom_minos,"trk_mom_minos/D");
  fTree->Branch("trk_charge_minos",&trk_charge_minos,"trk_charge_minos/D");
  fTree->Branch("trk_dcosx_minos",&trk_dcosx_minos,"trk_dcosx_minos/D");
  fTree->Branch("trk_dcosy_minos",&trk_dcosy_minos,"trk_dcosy_minos/D");
  fTree->Branch("trk_dcosz_minos",&trk_dcosz_minos,"trk_dcosz_minos/D");
  fTree->Branch("trk_vtxx_minos",&trk_vtxx_minos,"trk_vtxx_minos/D");
  fTree->Branch("trk_vtxy_minos",&trk_vtxy_minos,"trk_vtxy_minos/D");
  fTree->Branch("trk_vtxz_minos",&trk_vtxz_minos,"trk_vtxz_minos/D");
  fTree->Branch("trk_index_minos",&trk_index_minos,"trk_index_minos/I");
  fTree->Branch("mc_index_minos",&mc_index_minos,"mc_index_minos/F");
  fTree->Branch("mc_pdg_minos",&mc_pdg_minos,"mc_pdg_minos/D");
  fTree->Branch("mc_px_minos",&mc_px_minos,"mc_px_minos/D");
  fTree->Branch("mc_py_minos",&mc_py_minos,"mc_py_minos/D");
  fTree->Branch("mc_pz_minos",&mc_pz_minos,"mc_pz_minos/D");
  fTree->Branch("mc_ene_minos",&mc_ene_minos,"mc_ene_minos/D");
  fTree->Branch("mc_mass_minos",&mc_mass_minos,"mc_mass_minos/D");
  fTree->Branch("mc_vtxx_minos",&mc_vtxx_minos,"mc_vtxx_minos/D");
  fTree->Branch("mc_vtxy_minos",&mc_vtxy_minos,"mc_vtxy_minos/D");
  fTree->Branch("mc_vtxz_minos",&mc_vtxz_minos,"mc_vtxz_minos/D");   
  fTree->Branch("trkcontained_minos",&trkcontained_minos,"trkcontained_minos/I");  
  fTree->Branch("test_charge_minos",&test_charge_minos,"test_charge_minos/I"); 
  fTree->Branch("rdiff_minos",&rdiff_minos,"rdiff_minos/D");
  fTree->Branch("xdiff_minos",&xdiff_minos,"xdiff_minos/D");
  fTree->Branch("ydiff_minos",&ydiff_minos,"ydiff_minos/D");
  fTree->Branch("thetadiff_minos",&thetadiff_minos,"thetadiff_minos/D");
  fTree->Branch("parpdg", &parpdg, "parpdg/I");
  fTree->Branch("parmom", &parmom, "parmom/D");
  fTree->Branch("nuPDG_truth",&nuPDG_truth,"nuPDG_truth/I");
  fTree->Branch("ccnc_truth",&ccnc_truth,"ccnc_truth/I");
  fTree->Branch("mode_truth",&mode_truth,"mode_truth/I");
  fTree->Branch("enu_truth",&enu_truth,"enu_truth/D");
  fTree->Branch("Q2_truth",&Q2_truth,"Q2_truth/D");
  fTree->Branch("hitnuc_truth",&hitnuc_truth,"hitnuc_truth/I");
  fTree->Branch("W_truth",&W_truth,"W_truth/D");
  fTree->Branch("nuvtxx_truth",&nuvtxx_truth,"nuvtxx_truth/D");
  fTree->Branch("nuvtxy_truth",&nuvtxy_truth,"nuvtxy_truth/D");
  fTree->Branch("nuvtxz_truth",&nuvtxz_truth,"nuvtxz_truth/D");
  fTree->Branch("nu_dcosx_truth",&nu_dcosx_truth,"nu_dcosx_truth/D");
  fTree->Branch("nu_dcosy_truth",&nu_dcosy_truth,"nu_dcosy_truth/D");
  fTree->Branch("nu_dcosz_truth",&nu_dcosz_truth,"nu_dcosz_truth/D");
  fTree->Branch("lep_mom_truth",&lep_mom_truth,"lep_mom_truth/D");
  fTree->Branch("lep_dcosx_truth",&lep_dcosx_truth,"lep_dcosx_truth/D");
  fTree->Branch("lep_dcosy_truth",&lep_dcosy_truth,"lep_dcosy_truth/D");
  fTree->Branch("lep_dcosz_truth",&lep_dcosz_truth,"lep_dcosz_truth/D");
  fTree->Branch("mcevts_truth",&mcevts_truth,"mcevts_truth/I");
  fTree->Branch("beamwgt",&beamwgt,"beamwgt/D");
  fTree->Branch("tpx_flux",&tpx_flux,"tpx_flux/D");
  fTree->Branch("tpy_flux",&tpy_flux,"tpy_flux/D");
  fTree->Branch("tpz_flux",&tpz_flux,"tpz_flux/D");
  fTree->Branch("tptype_flux",&tptype_flux,"tptype_flux/I");
  fTree->Branch("nvertices",&nvertices,"nvertices/I");
  fTree->Branch("vtx_x",vtx_x,"vtx_x[nvertices]/D");
  fTree->Branch("vtx_y",vtx_y,"vtx_y[nvertices]/D");
  fTree->Branch("vtx_z",vtx_z,"vtx_z[nvertices]/D");
  fTree->Branch("vtx_w0",vtx_w0,"vtx_w0[nvertices]/D");
  fTree->Branch("vtx_w1",vtx_w1,"vtx_w1[nvertices]/D");
  fTree->Branch("vtx_t0",vtx_t0,"vtx_t0[nvertices]/D");
  fTree->Branch("vtx_t1",vtx_t1,"vtx_t1[nvertices]/D");

  //kinga:

  fTree->Branch("no_dead_wires_muon",&no_dead_wires_muon,"no_dead_wires_muon/I");  
  fTree->Branch("no_hits",&no_hits,"no_hits/I");
  fTree->Branch("hit_plane",hit_plane,"hit_plane[no_hits]/I");
  fTree->Branch("hit_wire",hit_wire,"hit_wire[no_hits]/I");
  fTree->Branch("hit_channel",hit_channel,"hit_channel[no_hits]/I");
  fTree->Branch("hit_peakT",hit_peakT,"hit_peakT[no_hits]/D");
  fTree->Branch("hit_peakTcorr",hit_peakTcorr,"hit_peakTcorr[no_hits]/D");
  fTree->Branch("hit_charge",hit_charge,"hit_charge[no_hits]/D");
  fTree->Branch("hit_ph",hit_ph,"hit_ph[no_hits]/D");
  fTree->Branch("hit_trkid",hit_trkid,"hit_trkid[no_hits]/I");
  fTree->Branch("hit_etruth",hit_etruth,"hit_etruth[no_hits]/D");
  fTree->Branch("no_clusters",&no_clusters,"no_clusters/I");
  fTree->Branch("clusters_planeNo",clusters_planeNo,"clusters_planeNo[no_clusters]/I");
  fTree->Branch("Start_pt_w_Cl", Start_pt_w_Cl, "Start_pt_w_Cl[no_clusters]/D");
  fTree->Branch("Start_pt_t_Cl", Start_pt_t_Cl, "Start_pt_t_Cl[no_clusters]/D");
  fTree->Branch("End_pt_w_Cl", End_pt_w_Cl, "End_pt_w_Cl[no_clusters]/D");
  fTree->Branch("End_pt_t_Cl", End_pt_t_Cl, "End_pt_t_Cl[no_clusters]/D");

  //fTree->Branch("no_fuzzyclusters",&no_fuzzyclusters,"no_fuzzyclusters/I");
  fTree->Branch("fNhitsClust",fNhitsClust,"fNhitsClust/I");

  fTree->Branch("two_trackstart_dcosx_reco",two_trackstart_dcosx_reco, "two_trackstart_dcosx_reco[2]/D");
  fTree->Branch("two_trackstart_dcosy_reco",two_trackstart_dcosy_reco, "two_trackstart_dcosy_reco[2]/D");
  fTree->Branch("two_trackstart_dcosz_reco",two_trackstart_dcosz_reco, "two_trackstart_dcosz_reco[2]/D");
  fTree->Branch("two_trackexit_dcosx_reco",two_trackexit_dcosx_reco, "two_trackexit_dcosx_reco[2]/D");
  fTree->Branch("two_trackexit_dcosy_reco",two_trackexit_dcosy_reco, "two_trackexit_dcosy_reco[2]/D");
  fTree->Branch("two_trackexit_dcosz_reco",two_trackexit_dcosz_reco, "two_trackexit_dcosz_reco[2]/D");
  
  //......................................................
  // from geant4:

  fTree->Branch("no_primaries",&no_primaries,"no_primaries/I");
  fTree->Branch("geant_list_size",&geant_list_size,"geant_list_size/I");
  
  fTree->Branch("pdg",pdg,"pdg[geant_list_size]/I");
  fTree->Branch("Eng",Eng,"Eng[geant_list_size]/D");
  fTree->Branch("Px",Px,"Px[geant_list_size]/D");
  fTree->Branch("Py",Py,"Py[geant_list_size]/D");
  fTree->Branch("Pz",Pz,"Pz[geant_list_size]/D");
  fTree->Branch("StartPointx",StartPointx,"StartPointx[geant_list_size]/D");
  fTree->Branch("StartPointy",StartPointy,"StartPointy[geant_list_size]/D");
  fTree->Branch("StartPointz",StartPointz,"StartPointz[geant_list_size]/D");
  fTree->Branch("EndPointx",EndPointx,"EndPointx[geant_list_size]/D");
  fTree->Branch("EndPointy",EndPointy,"EndPointy[geant_list_size]/D");
  fTree->Branch("EndPointz",EndPointz,"EndPointz[geant_list_size]/D");
  fTree->Branch("NumberDaughters",NumberDaughters,"NumberDaughters[geant_list_size]/I");
  fTree->Branch("Mother",Mother,"Mother[geant_list_size]/I");
  fTree->Branch("TrackId",TrackId,"TrackId[geant_list_size]/I");
  fTree->Branch("process_primary",process_primary,"process_primary[geant_list_size]/I");
  fTree->Branch("orig",orig,"orig[geant_list_size]/I");
      
  fTree->Branch("Kin_Eng_reco",Kin_Eng_reco,"Kin_Eng_reco[ntracks_reco]/D");
  //fTree->Branch("fTrkPitchC", &fTrkPitchC, "fTrkPitchC/D");
  fTree->Branch("muon_Kin_Eng_reco",&muon_Kin_Eng_reco,"muon_Kin_Eng_reco/D");
  
 //..................................
 // now from genie:
  fTree->Branch("genie_no_primaries",&genie_no_primaries,"genie_no_primaries/I");
  fTree->Branch("genie_primaries_pdg",genie_primaries_pdg,"genie_primaries_pdg[genie_no_primaries]/I");
  fTree->Branch("genie_Eng",genie_Eng,"genie_Eng[genie_no_primaries]/D");
  fTree->Branch("genie_Px",genie_Px,"genie_Px[genie_no_primaries]/D");
  fTree->Branch("genie_Py",genie_Py,"genie_Py[genie_no_primaries]/D");
  fTree->Branch("genie_Pz",genie_Pz,"genie_Pz[genie_no_primaries]/D");
  fTree->Branch("genie_P",genie_P,"genie_P[genie_no_primaries]/D");
  fTree->Branch("genie_status_code",genie_status_code,"genie_status_code[genie_no_primaries]/I");
  fTree->Branch("genie_mass",genie_mass,"genie_mass[genie_no_primaries]/D");
  fTree->Branch("genie_trackID",genie_trackID,"genie_trackID[genie_no_primaries]/I");
  fTree->Branch("genie_ND",genie_ND,"genie_ND[genie_no_primaries]/I");
  fTree->Branch("genie_mother",genie_mother,"genie_mother[genie_no_primaries]/I");
  //paddles
  fTree->Branch("pmttime",&pmttime,"pmttime/D");
  fTree->Branch("pmt1",pmt1,"pmt1[4]/I");
  fTree->Branch("pmt2",pmt2,"pmt2[4]/I");
  fTree->Branch("pmt3",pmt3,"pmt3[4]/I");
  fTree->Branch("pmt4",pmt4,"pmt4[4]/I");

}


void t962::AnalysisTreeT962::beginSubRun(const art::SubRun& sr)
{

  art::Handle< sumdata::POTSummary > potListHandle;
  //sr.getByLabel(fPOTModuleLabel,potListHandle);
  
  if(sr.getByLabel(fPOTModuleLabel,potListHandle))
    pot=potListHandle->totpot;
  else
    pot=0.;
  
}




void t962::AnalysisTreeT962::analyze(const art::Event& evt){
  
  ResetVars();
  
  run = evt.run();
  subrun = evt.subRun();
  event = evt.id().event();
  
  art::Timestamp ts = evt.time();
  TTimeStamp tts(ts.timeHigh(), ts.timeLow());
  evttime = tts.AsDouble();
  
  //copied from MergeDataPaddles.cxx
  art::Handle< raw::BeamInfo > beam;
  if (evt.getByLabel("beam",beam)){
    beamtime = (double)beam->get_t_ms();
    beamtime/=1000.; //in second
  }
  
  if (evt.isRealData()){
    isdata = 1;
  }
  else isdata = 0;
  
  art::Handle< std::vector<raw::RawDigit> > rdListHandle;
  evt.getByLabel(fDigitModuleLabel,rdListHandle);
  
  art::Handle< std::vector<sim::SimChannel> > scListHandle;
  evt.getByLabel(fDigitModuleLabel,scListHandle);
  
  art::Handle< std::vector<recob::Hit> > hitListHandle;
  std::vector<art::Ptr<recob::Hit> > hitlist;
  if (evt.getByLabel(fHitsModuleLabel,hitListHandle))
    art::fill_ptr_vector(hitlist, hitListHandle);
  
  art::Handle< std::vector<recob::Wire> > wireListHandle;
  evt.getByLabel(fCalDataModuleLabel,wireListHandle);
  
  art::Handle< std::vector<simb::MCTruth> > mctruthListHandle;
  std::vector<art::Ptr<simb::MCTruth> > mclist;
  if (evt.getByLabel(fGenieGenModuleLabel,mctruthListHandle))
    art::fill_ptr_vector(mclist, mctruthListHandle);
  
  art::Handle< std::vector<simb::MCFlux> > mcfluxListHandle;
  std::vector<art::Ptr<simb::MCFlux> > fluxlist;
  if (evt.getByLabel(fGenieGenModuleLabel,mcfluxListHandle))
    art::fill_ptr_vector(fluxlist, mcfluxListHandle);
  
  
  art::Handle< std::vector<recob::Cluster> > clusterListHandle;
  std::vector<art::Ptr<recob::Cluster> > clusterlist;
  if (evt.getByLabel(fClusterModuleLabel,clusterListHandle))
    art::fill_ptr_vector(clusterlist, clusterListHandle);
  
  art::Handle< std::vector<recob::Track> > trackListHandle;
  std::vector<art::Ptr<recob::Track> > tracklist;
  if (evt.getByLabel(fTrackModuleLabel,trackListHandle))
    art::fill_ptr_vector(tracklist, trackListHandle);
  
  art::Handle< std::vector<recob::EndPoint2D> > endpointListHandle;
  std::vector<art::Ptr<recob::EndPoint2D> > endpointlist;
  if (evt.getByLabel(fEndPoint2DModuleLabel,endpointListHandle))
    art::fill_ptr_vector(endpointlist, endpointListHandle);
  
  art::Handle< std::vector<recob::Vertex> > vertexListHandle;
  std::vector<art::Ptr<recob::Vertex> > vertexlist;
  if (evt.getByLabel(fVertexModuleLabel,vertexListHandle))
    art::fill_ptr_vector(vertexlist, vertexListHandle);

  art::Handle< std::vector<t962::MINOS> > minosListHandle;
  std::vector<art::Ptr<t962::MINOS> > minoslist;
  if (evt.getByLabel(fMINOSModuleLabel,minosListHandle))
    art::fill_ptr_vector(minoslist, minosListHandle);


  art::Handle<t962::Paddles> paddlesHandle;
  if (evt.getByLabel(fPaddlesModuleLabel,paddlesHandle)){
    pmttime = paddlesHandle->gettime();
    for (int i = 0; i<4; ++i){
      pmt1[i] = paddlesHandle->gettdc(0,i);
      pmt2[i] = paddlesHandle->gettdc(1,i);
      pmt3[i] = paddlesHandle->gettdc(2,i);
      pmt4[i] = paddlesHandle->gettdc(3,i);
    }
  }

  art::ServiceHandle<geo::Geometry> geom;  
  art::ServiceHandle<cheat::BackTracker> bt;
  auto const* detprop = lar::providerFrom<detinfo::DetectorPropertiesService>();
  //auto const* LArProp = lar::providerFrom<detinfo::LArPropertiesService>();
  
  // Electronic calibration factor to convert from ADC to electrons
  double fElectronsToADC = detprop->ElectronsToADC();
  
  double dethalfwidth = geom->DetHalfWidth();
  double dethalfheight = geom->DetHalfHeight();
  
  //vertex information
  if(vertexlist.size()){
    double vtxxyz[3];
    vertexlist[0]->XYZ(vtxxyz);
    vtxx_reco = vtxxyz[0];
    vtxy_reco = vtxxyz[1];
    vtxz_reco = vtxxyz[2];
  }
  nvertices = TMath::Min(int(vertexlist.size()),kMaxTrack);
  for(int i = 0; i < nvertices; ++i)
  {
    double vtxxyz[3];
    vertexlist[i]->XYZ(vtxxyz);
    vtx_x[i] = vtxxyz[0];
    vtx_y[i] = vtxxyz[1];
    vtx_z[i] = vtxxyz[2];
    vtx_w0[i] = geom->WireCoordinate(vtx_y[i],vtx_z[i],0,0,0);
    vtx_w1[i] = geom->WireCoordinate(vtx_y[i],vtx_z[i],1,0,0);
    vtx_t0[i] = detprop->ConvertXToTicks(vtx_x[i],0,0,0);
    vtx_t1[i] = detprop->ConvertXToTicks(vtx_x[i],1,0,0);

  }
  //cluster information
  nclusu_reco = 0;
  nclusv_reco = 0;
  nclusw_reco = 0;

  int nplanes = geom->Nplanes();
  std::vector< std::vector<int> > Cls(nplanes);

  for(size_t i=0; i<clusterlist.size();++i){
    
    switch(clusterlist[i]->View()){
    case geo::kU :
      nclusu_reco ++;
      Cls[0].push_back(i);
      break;
    case geo::kV :
      nclusv_reco ++;
      Cls[1].push_back(i);
      break;
    case geo::kW :
      nclusw_reco ++;
      Cls[2].push_back(i);
      break;
    default :
      break;
    }
  }

  no_clusters = clusterlist.size();
  for(size_t i=0; i<clusterlist.size();++i){
    if (clusterlist[i]->View()==geo::kU){
      clusters_planeNo[i]=0;
    }
    else if (clusterlist[i]->View()==geo::kV){
      clusters_planeNo[i]=1;
    } 
    Start_pt_w_Cl[i]=clusterlist[i]->StartWire();
    Start_pt_t_Cl[i]=clusterlist[i]->StartTick();
    End_pt_w_Cl[i]=clusterlist[i]->EndWire();
    End_pt_t_Cl[i]=clusterlist[i]->EndTick();
  }

  art::FindManyP<recob::Hit> fm(clusterListHandle, evt, fClusterModuleLabel);
  double mindistx = 1e10;
  double mindisty = 1e10;
  double mindistz = 1e10;
  for (size_t i = 0; i<Cls[0].size(); ++i){
    std::vector< art::Ptr<recob::Hit> > hitlist1 = fm.at(Cls[0][i]);
    for (auto ihit1 = hitlist1.begin(); ihit1!=hitlist1.end(); ++ihit1){
      double x0 = detprop->ConvertTicksToX((*ihit1)->PeakTime(),0,0,0);
      for (size_t j = 0; j<Cls[1].size(); ++j){
	std::vector< art::Ptr<recob::Hit> > hitlist2 = fm.at(Cls[1][j]);
	for (auto ihit2 = hitlist2.begin(); ihit2!=hitlist2.end(); ++ihit2){
	  double x1 = detprop->ConvertTicksToX((*ihit2)->PeakTime(),1,0,0);
	  if (std::abs(x0-x1)<0.5){
	    double y,z;
	    if (geom->ChannelsIntersect((*ihit1)->Channel(),
					(*ihit2)->Channel(),
					y,z)){
	      if (std::abs(x0)<mindistx){
		mindistx = std::abs(x0);
		nearestx[0] = x0;
		nearestx[1] = y;
		nearestx[2] = z;
	      }
	      if (std::abs(x0-2*dethalfwidth)<mindistx){
		mindistx = std::abs(x0-2*dethalfwidth);
		nearestx[0] = x0;
		nearestx[1] = y;
		nearestx[2] = z;
	      }
	      if (std::abs(y-dethalfheight)<mindisty){
		mindisty = std::abs(y-dethalfheight);
		nearesty[0] = x0;
		nearesty[1] = y;
		nearesty[2] = z;
	      }
	      if (std::abs(z)<mindistz){
		mindistz = std::abs(z);
		nearestz[0] = x0;
		nearestz[1] = y;
		nearestz[2] = z;
	      }
	    }
	  }
	}
      }
    }
  }
	    
  for (int i = 0; i<nplanes; i++){
    int n_vertexclusters = 0;
    if (Cls[i].size()>0){
      int vtx2d_w = -99999;
      double vtx2d_t = -99999;
      bool found2dvtx = false;
      //find 2d vertex
      for (unsigned int j = 0; j<endpointlist.size();j++){
	if (endpointlist[j]->View() == clusterlist[Cls[i][0]]->View()){
	  vtx2d_w = endpointlist[j]->WireID().Wire;
	  vtx2d_t = endpointlist[j]->DriftTime();
	  found2dvtx = true;
	  break;
	}
      }
      if (found2dvtx){
	for (unsigned j = 0; j<Cls[i].size(); j++){
	  double w = clusterlist[Cls[i][j]]->StartWire();
	  double t = clusterlist[Cls[i][j]]->StartTick();
	  double dtdw = std::tan(clusterlist[Cls[i][j]]->StartAngle());
	  double t_vtx = t+dtdw*(vtx2d_w-w);
	  double dis = TMath::Abs(vtx2d_t-t_vtx);
	  if (dis<fvertexclusterWindow){
	    n_vertexclusters++;
	  }
	}
      }
    }
    if (i==0){
      nvertexclustersu_reco = n_vertexclusters;
    }
    else if (i==1){
      nvertexclustersv_reco = n_vertexclusters;
    }
    else if (i==2){
      nvertexclustersw_reco = n_vertexclusters;
    }
  }

  
  test_charge_minos=0.;
  for(unsigned int j = 0; j < minoslist.size(); j++){ 
    if (!isdata){
      if(minoslist[j]->fcharge<0.)
	test_charge_minos=-1;
    }
  }
  
  //track information
  ntracks_reco=tracklist.size();
  double larStart[3];
  double larEnd[3];
  std::vector<double> trackStart;
  std::vector<double> trackEnd;
  //grab information about whether a track is associated with a vertex and whether a track leaves the detector. these variables are powerful discriminators.
  int n_vertextracks=0;
  int n_endonboundarytracks=0;
  art::FindManyP<anab::Calorimetry> fmcal(trackListHandle, evt, fCalorimetryModuleLabel);
  art::FindManyP<anab::ParticleID>  fmpid(trackListHandle, evt, fParticleIDModuleLabel);
  art::FindManyP<recob::Hit>        fmht(trackListHandle, evt, fTrackModuleLabel);
  art::FindManyP<recob::Track>      fmtk(hitListHandle, evt, fTrackModuleLabel);
  art::FindManyP<recob::Track>      fmtkcl(clusterListHandle, evt, fTrackModuleLabel);
  //art::FindManyP<recob::Cluster>    fmcltk(trackListHandle, evt, fTrackModuleLabel);
  nclupertrack_reco = 0;
  if (fmtkcl.isValid()){
    for(size_t i=0; i<clusterlist.size();++i){
      if (int(fmtkcl.at(i).size())>nclupertrack_reco ){
	nclupertrack_reco = fmtkcl.at(i).size();
      }
    }
  }
  for(unsigned int i=0; i<tracklist.size();++i){
    trackStart.clear();
    trackEnd.clear();
    memset(larStart, 0, 3);
    memset(larEnd, 0, 3);
    tracklist[i]->Extent(trackStart,trackEnd); 
    tracklist[i]->Direction(larStart,larEnd);
    
    //kinga:
    if(tracklist.size()==2){ // (2 track event)
      two_trackstart_dcosx_reco[i] = larStart[0];
      two_trackstart_dcosy_reco[i] = larStart[1];
      two_trackstart_dcosz_reco[i] = larStart[2];       
      two_trackexit_dcosx_reco[i] = larEnd[0];
      two_trackexit_dcosy_reco[i] = larEnd[1];
      two_trackexit_dcosz_reco[i] = larEnd[2];
    }
    
    trackstart_x_reco=trackStart[0];
    trackstart_y_reco=trackStart[1];
    trackstart_z_reco=trackStart[2];
    trackexit_x_reco=trackEnd[0];
    trackexit_y_reco=trackEnd[1];
    trackexit_z_reco=trackEnd[2];  
    if (!isdata&&mclist.size()){
      if (mclist[0]->NeutrinoSet()){
	if(sqrt(pow(trackstart_x_reco-mclist[0]->GetNeutrino().Nu().Vx(),2)+pow(trackstart_y_reco-mclist[0]->GetNeutrino().Nu().Vy(),2)+pow(trackstart_z_reco-mclist[0]->GetNeutrino().Nu().Vz(),2))<fvertextrackWindow)
	  n_vertextracks++; 
      }
    }
    if(EndsOnBoundary(tracklist[i])) n_endonboundarytracks++;
    trkvtxx[i]        = trackStart[0];
    trkvtxy[i]        = trackStart[1];
    trkvtxz[i]        = trackStart[2];
    trkendx[i]        = trackEnd[0];
    trkendy[i]        = trackEnd[1];
    trkendz[i]        = trackEnd[2];
    trkstartdcosx[i]  = larStart[0];
    trkstartdcosy[i]  = larStart[1];
    trkstartdcosz[i]  = larStart[2];
    trkenddcosx[i]    = larEnd[0];
    trkenddcosy[i]    = larEnd[1];
    trkenddcosz[i]    = larEnd[2];
    if (fmcal.isValid()){
      for (size_t j = 0; j<fmcal.at(i).size(); ++j){
	if (!fmcal.at(i)[j]->PlaneID().isValid) continue;
	int planenum = fmcal.at(i)[j]->PlaneID().Plane;
	if (planenum<0||planenum>1) continue;
	trkke[i][planenum]           = fmcal.at(i)[j]->KineticEnergy();
	trkrange[i][planenum]        = fmcal.at(i)[j]->Range();
	trkpitchc[i][planenum]       = fmcal.at(i)[j]->TrkPitchC();
	no_dead_wires_muon = fmcal.at(i)[j]->DeadWireResRC().size();
	Kin_Eng_reco[i][planenum]    = fmcal.at(i)[j]->KineticEnergy()+no_dead_wires_muon*fmcal.at(i)[j]->TrkPitchC()*2.1;
	ntrkhits[i][planenum]        = fmcal.at(i)[j]->dEdx().size();
	if (nmaxtrkhits[planenum]<ntrkhits[i][j]) nmaxtrkhits[planenum] = ntrkhits[i][j];
	for (int k = 0; k<ntrkhits[i][planenum]; ++k){
	  trkdedx[i][planenum][k] = (fmcal.at(i)[j]->dEdx())[k];
	  trkdqdx[i][planenum][k] = (fmcal.at(i)[j]->dQdx())[k];
	  trkresrg[i][planenum][k] = (fmcal.at(i)[j]->ResidualRange())[k];
	}
      }
    }
    if (fmpid.isValid()){
      for (size_t j = 0; j<fmpid.at(i).size(); ++j){
	if (!fmpid.at(i)[j]->PlaneID().isValid) continue;
	int planenum = fmpid.at(i)[j]->PlaneID().Plane;
	if (planenum<0||planenum>1) continue;
	trkpid[i][planenum]         = fmpid.at(i)[j]->Pdg();
	trkpidndf[i][planenum]      = fmpid.at(i)[j]->Ndf();
	trkpidchi2[i][planenum]     = fmpid.at(i)[j]->MinChi2();
	trkpida[i][planenum]        = fmpid.at(i)[j]->PIDA();
	trkmissinge[i][planenum]    = fmpid.at(i)[j]->MissingE();
	trkmissingeavg[i][planenum] = fmpid.at(i)[j]->MissingEavg();
      }
    }
    if (!isdata){
      // get the hits in each view
      std::vector< art::Ptr<recob::Hit> > allHits = fmht.at(i);
      std::vector< art::Ptr<recob::Hit> > hitsU;
      std::vector< art::Ptr<recob::Hit> > hitsV;
      for(size_t ah = 0; ah < allHits.size(); ++ah){
	if     (allHits[ah]->View() == geo::kU) hitsU.push_back(allHits[ah]);
	else if(allHits[ah]->View() == geo::kV) hitsV.push_back(allHits[ah]);
      }
      int trkid;
      double purity;
      double maxe;
      HitsPurity(hitsU, trkid, purity, maxe);
      trktruepuru[i] = purity;
      if (trkid>0){
	const simb::MCParticle *particle = bt->TrackIDToParticle(trkid);
	const std::vector<sim::IDE> vide = bt->TrackIDToSimIDE(trkid);
	double tote = 0;
	for (size_t iide = 0; iide<vide.size(); ++iide){
	  tote += vide[iide].energy;
	}
	trktruepdgu[i] = particle->PdgCode();
	trktrueeffu[i] = maxe/(tote/2); //I believe tote include both induction and collection energies
      }
      HitsPurity(hitsV, trkid, purity, maxe);
      trktruepurv[i] = purity;
      if (trkid>0){
	const simb::MCParticle *particle = bt->TrackIDToParticle(trkid);
	const std::vector<sim::IDE> vide = bt->TrackIDToSimIDE(trkid);
	double tote = 0;
	for (size_t iide = 0; iide<vide.size(); ++iide){
	  tote += vide[iide].energy;
	}
	trktruepdgv[i] = particle->PdgCode();
	trktrueeffv[i] = maxe/(tote/2);
      }
      HitsPurity(allHits, trkid, purity, maxe);
      trkg4Id[i] = trkid;
    }
  }
  nvertextracks_reco=n_vertextracks; 
  ntrackendonboundary_reco=n_endonboundarytracks;
  
  //////////////////////////////////////////////////////////////////////////
  /////////        HIT info:       ////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////
  
  std::vector< art::Ptr<recob::Hit> >::iterator itr = hitlist.begin();
  //unsigned int p(0),w(0),t(0), cs(0);
  int hit_no;
  unsigned int channel=0;
  
  //std::cout<<"hitlist.size()= "<<hitlist.size()<<std::endl;
  
  no_hits=hitlist.size();
  
  while( itr != hitlist.end() ){
    // std::cout<<"working on hit# "<<itr-hitlist.begin()<<std::endl;
    hit_no=(int)(itr-hitlist.begin());
    //std::cout<<"hit_no= "<<hit_no<<std::endl;
    //std::cout<<"channel= "<<(*itr)->Channel()<<" ";
    channel=(*itr)->Channel();
    
    std::vector<geo::WireID> wireids = geom->ChannelToWire(channel);
    hit_channel[hit_no]= channel;
    hit_plane[hit_no]=wireids[0].Plane;
    hit_wire[hit_no]=wireids[0].Wire;
    hit_peakT[hit_no]=(*itr)->PeakTime();
    hit_peakTcorr[hit_no]=(*itr)->PeakTime()-detprop->GetXTicksOffset(wireids[0].Plane,0,0);
    hit_charge[hit_no]=(*itr)->Integral();
    hit_ph[hit_no] = (*itr)->PeakAmplitude();
    if (fmtk.at(hit_no).size()!=0){
      hit_trkid[hit_no] = fmtk.at(hit_no)[0]->ID();
    }
    
    if (!isdata){
      std::vector<sim::TrackIDE> eveIDs = bt->HitToEveID(*itr);
      double etruth = 0;
      for (size_t e = 0; e < eveIDs.size(); ++e){
	etruth += eveIDs[e].energy;
      }
      hit_etruth[hit_no] = etruth;
    }
    itr++;
  }
  
  //mc truth information
  if (!isdata){
    //save single particle information
    art::ServiceHandle<cheat::BackTracker> bt;
    const sim::ParticleList& plist = bt->ParticleList();
    for ( sim::ParticleList::const_iterator ipar = plist.begin(); ipar!=plist.end(); ++ipar){
      simb::MCParticle *particle = ipar->second;
      parpdg = particle->PdgCode();
      parmom = particle->Momentum().P();
      break;
    }
    //--------------------------------------------------------------//
    //        NOW I WILL GET INFO FROM GEANT4 TO FIND OUT HOW MANY 
    //        PARTICLES WE CAN REALLY SEE IN OUR DETECTOR
    //        this is needed if you want to confirm that kingaclusters 
    ///       can correctly count tracks:
    //--------------------------------------------------------------//
    
    std::vector<const simb::MCParticle* > geant_part;
    for(size_t p = 0; p < plist.size(); ++p) geant_part.push_back(plist.Particle(p)); 
    
    //std::cout<<"No of geant part= "<<geant_part.size()<<std::endl;
    std::string pri("primary");
    int primary=0;
    int geant_particle=0;
    //determine the number of primary particles from geant:
    
    for( unsigned int i = 0; i < geant_part.size(); ++i ){
      geant_particle++;
      if(geant_part[i]->Process()==pri){
        primary++;
      }
    }
    
    no_primaries=primary;
    geant_list_size=geant_particle;
    
    //std::cout<<"Geant4 list: "<<std::endl;
    
    for( unsigned int i = 0; i < geant_part.size(); ++i ){
      
      
      if(geant_part[i]->Process()==pri){
        process_primary[i]=1;
      }
      else{
        process_primary[i]=0;
      }
      
      Mother[i]=geant_part[i]->Mother();
      TrackId[i]=geant_part[i]->TrackId();
      
      pdg[i]=geant_part[i]->PdgCode();
      
      Eng[i]=geant_part[i]->E();
      Px[i]=geant_part[i]->Px();
      
      Py[i]=geant_part[i]->Py();
      Pz[i]=geant_part[i]->Pz();
      
      StartPointx[i]=geant_part[i]->Vx();
      StartPointy[i]=geant_part[i]->Vy();
      StartPointz[i]=geant_part[i]->Vz();
      EndPointx[i]=geant_part[i]->EndPosition()[0];
      EndPointy[i]=geant_part[i]->EndPosition()[1];
      EndPointz[i]=geant_part[i]->EndPosition()[2];
      
      NumberDaughters[i]=geant_part[i]->NumberDaughters();

      auto & mct  = bt->ParticleToMCTruth(geant_part[i]);
      if (mct){
        orig[i] = mct->Origin();
      }
    } //geant particles
    //beam weight, only available for nu-data
    beamwgt = 1.;
    
    //save neutrino interaction information
    mcevts_truth = mclist.size();
    if (mcevts_truth){//at least one mc record
      art::Ptr<simb::MCTruth> mctruth = mclist[0];
      if (mctruth->NeutrinoSet()){//is neutrino
	
        nuPDG_truth = mctruth->GetNeutrino().Nu().PdgCode();
        ccnc_truth = mctruth->GetNeutrino().CCNC();
        mode_truth = mctruth->GetNeutrino().Mode();
        Q2_truth = mctruth->GetNeutrino().QSqr();
        W_truth = mctruth->GetNeutrino().W();
        hitnuc_truth = mctruth->GetNeutrino().HitNuc();
        enu_truth = mctruth->GetNeutrino().Nu().E();
        nuvtxx_truth = mctruth->GetNeutrino().Nu().Vx();
        nuvtxy_truth = mctruth->GetNeutrino().Nu().Vy();
        nuvtxz_truth = mctruth->GetNeutrino().Nu().Vz();
        if (mctruth->GetNeutrino().Nu().P()){
          nu_dcosx_truth = mctruth->GetNeutrino().Nu().Px()/mctruth->GetNeutrino().Nu().P();
          nu_dcosy_truth = mctruth->GetNeutrino().Nu().Py()/mctruth->GetNeutrino().Nu().P();
          nu_dcosz_truth = mctruth->GetNeutrino().Nu().Pz()/mctruth->GetNeutrino().Nu().P();
        }
        lep_mom_truth = mctruth->GetNeutrino().Lepton().P();
        if (mctruth->GetNeutrino().Lepton().P()){
          lep_dcosx_truth = mctruth->GetNeutrino().Lepton().Px()/mctruth->GetNeutrino().Lepton().P();
          lep_dcosy_truth = mctruth->GetNeutrino().Lepton().Py()/mctruth->GetNeutrino().Lepton().P();
          lep_dcosz_truth = mctruth->GetNeutrino().Lepton().Pz()/mctruth->GetNeutrino().Lepton().P();
        }
	if (fluxlist.size()){
	  art::Ptr<simb::MCFlux>  mcflux = fluxlist[0];
	  
	  tpx_flux = mcflux->ftpx;
	  tpy_flux = mcflux->ftpy;
	  tpz_flux = mcflux->ftpz;
	  tptype_flux = mcflux->ftptype;
	}	
      }//is neutrino
      genie_no_primaries=mctruth->NParticles();
      
      for(int j = 0; j < mctruth->NParticles(); ++j){
        simb::MCParticle part(mctruth->GetParticle(j));
	
        genie_primaries_pdg[j]=part.PdgCode();
        genie_Eng[j]=part.E();
        genie_Px[j]=part.Px();
        genie_Py[j]=part.Py();
        genie_Pz[j]=part.Pz();
        genie_P[j]=part.P();
        genie_status_code[j]=part.StatusCode();
        genie_mass[j]=part.Mass();
        genie_trackID[j]=part.TrackId();
        genie_ND[j]=part.NumberDaughters();
        genie_mother[j]=part.Mother();
      }
    }//at least one mc record
  }//mc
  //minos matching information
  //  try{
  //find matched MINOS information for each track
  nmatched_reco = 0;
  art::FindManyP<t962::MINOS> fmmatch(trackListHandle, evt, fTrackMatchModuleLabel);
  
  //grab information about where track started and ended and the dcos at those points
  int bestmatch = -1;
  int bestmatchminos = -1;
  double totaldiffmin = 1e10;
  if (fmmatch.isValid()){
    for(size_t i=0; i<tracklist.size();++i){
      if (!fmmatch.at(i).size()) continue;
      
      ++nmatched_reco;

      trackStart.clear();
      trackEnd.clear();
      memset(larStart, 0, 3);
      memset(larEnd, 0, 3);
      
      tracklist[i]->Direction(larStart,larEnd);
      tracklist[i]->Extent(trackStart,trackEnd);  

      //Redefine direction using track start/end point
      larEnd[0] = (trackEnd[0]-trackStart[0])/sqrt(pow(trackEnd[0]-trackStart[0],2)+pow(trackEnd[1]-trackStart[1],2)+pow(trackEnd[2]-trackStart[2],2));
      larEnd[1] = (trackEnd[1]-trackStart[1])/sqrt(pow(trackEnd[0]-trackStart[0],2)+pow(trackEnd[1]-trackStart[1],2)+pow(trackEnd[2]-trackStart[2],2));
      larEnd[2] = (trackEnd[2]-trackStart[2])/sqrt(pow(trackEnd[0]-trackStart[0],2)+pow(trackEnd[1]-trackStart[1],2)+pow(trackEnd[2]-trackStart[2],2));

      if (trackStart[2]>trackEnd[2]){//flip direction
	trackStart.swap(trackEnd);
	for (size_t j = 0; j<3; ++j){
	  double tmp = larStart[j];
	  larStart[j] = -larEnd[j];
	  larEnd[j] = -tmp;
	}
      }
      
      for (size_t j = 0; j<fmmatch.at(i).size(); ++j){
	
	double D=(90*0.5)+(42.4*2.54)-5.588; //distance from the front (upstream) of the TPC to the 1st Minos plane 
	//(this minus number is the one we measured with Mitch)
	
	double x_offset=117.4; // previously 116.9;
	double y_offset=19.3; // previously  20.28;
	
	if (isdata){
	  x_offset += -3.134 + 1.867576;
	  y_offset += -0.837;
	}
	else{
	  x_offset += -0.893;
	  y_offset += -1.597 + 2;
	}
	
	
	double dz = D - trackEnd[2]+(100.0 * fmmatch.at(i)[j]->ftrkVtxZ);//z-difference between end of T962 track and
	//begin of MINOS track...in centimeters
	
	double l = dz/(larEnd[2]);//3-d distance between end of T962 track and begin of MINOS track
	
	double x_pred = l*larEnd[0]+trackEnd[0];//predicted x-pos. of T962 track at z-position equal to
	//start of MINOS track
	double y_pred = l*larEnd[1]+trackEnd[1];//predicted y-pos. of T962 track at z-position equal to
	//start of MINOS track
	
	double dx = 100.0*fmmatch.at(i)[j]->ftrkVtxX - x_offset - x_pred;
	double dy = 100.0*fmmatch.at(i)[j]->ftrkVtxY + y_offset - y_pred;
	
	double rdiff = sqrt(dx*dx + dy*dy);
	double thetadiff = TMath::ACos((larEnd[0]*fmmatch.at(i)[j]->ftrkdcosx)+(larEnd[1]*fmmatch.at(i)[j]->ftrkdcosy)+(larEnd[2]*fmmatch.at(i)[j]->ftrkdcosz));
	//totaldiff is a measure of the agreement between the ArgoNeuT projected track and the MINOS track based on radial distance and angle. totaldiff= rdiff/cos(theta)  
	double totaldiff=fabs(rdiff/((larEnd[0]*fmmatch.at(i)[j]->ftrkdcosx)+(larEnd[1]*fmmatch.at(i)[j]->ftrkdcosy)+(larEnd[2]*fmmatch.at(i)[j]->ftrkdcosz)));
	if (totaldiff<totaldiffmin){
	  totaldiffmin = totaldiff;
	  bestmatch = i;
	  bestmatchminos = j;
	  rdiff_minos = rdiff;
	  thetadiff_minos = thetadiff;
	  xdiff_minos = dx;
	  ydiff_minos = dy;
	  //std::cout<<i<<" "<<rdiff<<" "<<thetadiff<<" "<<dx<<" "<<dy<<" "<<totaldiff<<" "<<bestmatch<<std::endl;
	}
      }
    }

    if (bestmatch>-1&&bestmatchminos>-1){
      trk_index_minos = bestmatch;
      trackStart.clear();
      trackEnd.clear();
      memset(larStart, 0, 3);
      memset(larEnd, 0, 3);

      if (trackStart[2]>trackEnd[2]){//flip direction
	trackStart.swap(trackEnd);
	for (size_t j = 0; j<3; ++j){
	  double tmp = larStart[j];
	  larStart[j] = -larEnd[j];
	  larEnd[j] = -tmp;
	}
      }
    
      tracklist[bestmatch]->Direction(larStart,larEnd);
      tracklist[bestmatch]->Extent(trackStart,trackEnd);  
    
      trackstart_dcosx_reco = larStart[0];
      trackstart_dcosy_reco = larStart[1];
      trackstart_dcosz_reco = larStart[2];       
      trackexit_dcosx_reco = larEnd[0];
      trackexit_dcosy_reco = larEnd[1];
      trackexit_dcosz_reco = larEnd[2];
      
      trackstart_x_reco=trackStart[0];
      trackstart_y_reco=trackStart[1];
      trackstart_z_reco=trackStart[2];
      trackexit_x_reco=trackEnd[0];
      trackexit_y_reco=trackEnd[1];
      trackexit_z_reco=trackEnd[2];  
      
      if (vtxx_reco<-100){
	vtxx_reco = trackstart_x_reco;
	vtxy_reco = trackstart_y_reco;
	vtxz_reco = trackstart_z_reco;
      }
    if(fmmatch.at(bestmatch)[bestmatchminos]->ftrkcontained)
      trk_mom_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkErange;
    else
      trk_mom_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkmom;     
    
    trkcontained_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkcontained; 
    trk_charge_minos = fmmatch.at(bestmatch)[bestmatchminos]->fcharge;
    trk_dcosx_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkdcosx;
    trk_dcosy_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkdcosy;
    trk_dcosz_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkdcosz;
    trk_vtxx_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkVtxX;
    trk_vtxy_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkVtxY;
    trk_vtxz_minos = fmmatch.at(bestmatch)[bestmatchminos]->ftrkVtxZ;        
    
    if (Kin_Eng_reco[1][bestmatch]>0 )
      muon_Kin_Eng_reco = Kin_Eng_reco[1][bestmatch];
    else if (Kin_Eng_reco[0][bestmatch]>0 )
      muon_Kin_Eng_reco = Kin_Eng_reco[0][bestmatch];
    
    if (!isdata){       
      mc_index_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcIndex;
      mc_pdg_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcPDG;
      mc_px_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcPx;
      mc_py_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcPy;
      mc_pz_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcPz;
      mc_ene_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcEne;
      mc_mass_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcMass;
      mc_vtxx_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcVtxX;
      mc_vtxy_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcVtxY;
      mc_vtxz_minos = fmmatch.at(bestmatch)[bestmatchminos]->fmcVtxZ;
    }
    
    //Calculate event energy
    enu_reco = 0;
    //sum the energy of hits not associated with tracks
    for(size_t i=0; i<hitlist.size();++i){
      if (hitlist[i]->View() == geo::kV){//collection view
	//std::cout<<i<<" "<<fmtk.at(i).size()<<std::endl;
	if (fmtk.at(i).size()==0){//hit not on any tracks
	  double time   = hitlist[i]->PeakTime();
	  double MIPs   = hitlist[i]->PeakAmplitude();   // in ADC
	  double WirePitch = geom->WirePitch(0,1,1);
	  double dQdx = MIPs/WirePitch;
	  double dQdx_e = dQdx/fElectronsToADC;  // Conversion from ADC/cm to e/cm
	  
	  dQdx_e *= LifetimeCorrection(time);   // Lifetime Correction (dQdx_e in e/cm)
	  
          // Note: LArProperties::BirksCorrection() has moved to DetectorProperties/DetectorPropertiesService
	  double dEdx = detprop->BirksCorrection(dQdx_e);   // Correction for charge quenching (Recombination) dEdx in MeV/cm
	  enu_reco += dEdx*WirePitch;
	}
      }
    }
    enu_reco /= 1000;
    //enu_reco = enu_reco + MPi; //assuming pion mass
    
      //sum track energy except the muon
    for(size_t i=0; i<tracklist.size();++i){
      if (int(i)!=bestmatch){ //not the matched track
	//	  double mass = MPi;  //default is pion mass
	//	  if (trkpid[i] == 2212
	//	      &&trkendx[i]>3&&trkendx[i]<44
	//	      &&trkendy[i]>-16&&trkendy[i]<16
	//	      &&trkendz[i]>6&&trkendz[i]<86) mass = MPro; //proton 
	if (trkke[1][i]>0){
	  enu_reco += trkke[1][i]/1000;
	}
	else if (trkke[0][i]>0){
	  enu_reco += trkke[0][i]/1000;
	}
      }
    }
    
    //add muon energy
    enu_reco += sqrt(pow(trk_mom_minos,2)+pow(MMu,2));
    
    }//if bestmatch>-1
  }
   // Note: LArProperties::ElectronLifetime() has moved to DetectorProperties/DetectorPropertiesService
  taulife = detprop->ElectronLifetime();
  fTree->Fill();
}

void t962::AnalysisTreeT962::HitsPurity(std::vector< art::Ptr<recob::Hit> > const& hits, int& trackid, double& purity, double& maxe){

  trackid = -1;
  purity = -1;
  
  art::ServiceHandle<cheat::BackTracker> bt;
  
  std::map<int,double> trkide;

  for(size_t h = 0; h < hits.size(); ++h){

    art::Ptr<recob::Hit> hit = hits[h];
    std::vector<sim::IDE> ides;
    //bt->HitToSimIDEs(hit,ides);
    std::vector<sim::TrackIDE> eveIDs = bt->HitToEveID(hit);

    for(size_t e = 0; e < eveIDs.size(); ++e){
      //std::cout<<h<<" "<<e<<" "<<eveIDs[e].trackID<<" "<<eveIDs[e].energy<<" "<<eveIDs[e].energyFrac<<std::endl;
      trkide[eveIDs[e].trackID] += eveIDs[e].energy;
    }
  }

  maxe = -1;
  double tote = 0;
  for (std::map<int,double>::iterator ii = trkide.begin(); ii!=trkide.end(); ++ii){
    tote += ii->second;
    if ((ii->second)>maxe){
      maxe = ii->second;
      trackid = ii->first;
    }
  }
  
  
  if (tote>0){
    purity = maxe/tote;
  }
}

  //---------------------------------------------------------------- 
void t962::AnalysisTreeT962::ResetVars(){

  run = -99999;
  subrun = -99999;
  event = -99999;
  evttime = -99999;
  beamtime = -99999;
  isdata = -99999;
  taulife = -99999;
  vtxx_reco = -99999;
  vtxy_reco  = -99999;
  vtxz_reco  = -99999;
  nclusu_reco  = -99999;
  nclusv_reco  = -99999;
  nclusw_reco  = -99999;
  ntracks_reco = -99999;
  nvertextracks_reco = -99999;
  nvertexclustersu_reco = -99999;
  nvertexclustersv_reco = -99999;
  nvertexclustersw_reco = -99999;
  for (int i = 0; i<3; ++i){
    nearestx[i] = -99999;
    nearesty[i] = -99999;
    nearestz[i] = -99999;
  }
  trackstart_x_reco = -99999;
  trackstart_y_reco = -99999;
  trackstart_z_reco = -99999;
  trackexit_x_reco = -99999;
  trackexit_y_reco = -99999;
  trackexit_z_reco = -99999;  
  trackstart_dcosx_reco = -99999;
  trackstart_dcosy_reco = -99999;
  trackstart_dcosz_reco = -99999;       
  trackexit_dcosx_reco = -99999;
  trackexit_dcosy_reco = -99999;
  trackexit_dcosz_reco = -99999;
  enu_reco = -99999;
  nclupertrack_reco = -99999;
  nmatched_reco = -99999;
  trk_mom_minos = -99999;
  trk_charge_minos = -99999;
  trk_dcosx_minos = -99999;
  trk_dcosy_minos = -99999;
  trk_dcosz_minos = -99999;  
  trk_vtxx_minos = -99999;
  trk_vtxy_minos = -99999;
  trk_vtxz_minos = -99999;
  trk_index_minos= -99999;
  muon_Kin_Eng_reco = -99999;
  no_dead_wires_muon = -99999;
  mc_index_minos=-99999;
  mc_pdg_minos=-99999;
  mc_px_minos=-99999;
  mc_py_minos=-99999;
  mc_pz_minos=-99999;
  mc_ene_minos=-99999;
  mc_mass_minos=-99999;
  mc_vtxx_minos=-99999;
  mc_vtxy_minos=-99999;
  mc_vtxz_minos=-99999;
  test_charge_minos=-99999;
  trkcontained_minos=-99999;
  rdiff_minos = 99999;
  xdiff_minos = 99999;
  ydiff_minos = 99999;
  thetadiff_minos = 99999;
//  vtxx_scan=-99999;
//  vtxy_scan=-99999;
//  vtxz_scan=-99999;
//  neutrino_scan=-99999;
//  maybeneutrino_scan=-99999;
//  ntracks_scan=-99999;
//  nshowers_scan=-99999;         
  parpdg = -99999;
  parmom = -99999;
  nuPDG_truth = -99999;
  ccnc_truth = -99999;
  mode_truth = -99999;
  enu_truth = -99999;
  Q2_truth = -99999;
  W_truth = -99999;
  hitnuc_truth = -99999;
  nuvtxx_truth = -99999;
  nuvtxy_truth = -99999;
  nuvtxz_truth = -99999;
  nu_dcosx_truth = -99999;
  nu_dcosy_truth = -99999;
  nu_dcosz_truth = -99999;
  lep_mom_truth = -99999;
  lep_dcosx_truth = -99999;
  lep_dcosy_truth = -99999;
  lep_dcosz_truth = -99999;
  mcevts_truth = -99999;
  beamwgt = -99999;
  tpx_flux = -99999;
  tpy_flux = -99999;
  tpz_flux = -99999;
  tptype_flux = -99999;
  nvertices = -99999;

  for (int i = 0; i<2; ++i) nmaxtrkhits[i] = -99999;
  for (int i = 0; i < kMaxTrack; i++){
    trkvtxx[i] = -99999;
    trkvtxy[i] = -99999;
    trkvtxz[i] = -99999;
    trkendx[i] = -99999;
    trkendy[i] = -99999;
    trkendz[i] = -99999;
    vtx_x[i] = -99999;
    vtx_y[i] = -99999;
    vtx_z[i] = -99999;
    vtx_w0[i] = -99999;
    vtx_w1[i] = -99999;
    vtx_t0[i] = -99999;
    vtx_t1[i] = -99999;
    trkstartdcosx[i] = -99999;
    trkstartdcosy[i] = -99999;
    trkstartdcosz[i] =  -99999;
    trkenddcosx[i] = -99999;
    trkenddcosy[i] = -99999;
    trkenddcosz[i] = -99999;
    for (int j = 0; j<2; ++j){
      trkke[i][j] = -99999;
      Kin_Eng_reco[i][j] = -99999;
      trkrange[i][j] = -99999;
      trkpitchc[i][j] = -99999;
      ntrkhits[i][j] = -99999;
      for (int k = 0; k<kMaxTrackHits; ++k){
	trkdedx[i][j][k] = -99999;
	trkdqdx[i][j][k] = -99999;
	trkresrg[i][j][k] = -99999;
	trkhitindexc[i][j][k] = -99999;
      }
      trkpid[i][j] = -99999;
      trkpidndf[i][j] = -99999;
      trkpidchi2[i][j] = -99999;
      trkpida[i][j] = -99999;
      trkmissinge[i][j] = -99999;
      trkmissingeavg[i][j] = -99999;
    }
    trktruepdgu[i] = -99999;
    trktrueeffu[i] = -99999;
    trktruepuru[i] = -99999;
    trktruepdgv[i] = -99999;
    trktrueeffv[i] = -99999;
    trktruepurv[i] = -99999;
    trkg4Id[i] = -99999;
  }    
  
  no_primaries = -99999;
  geant_list_size=-999;
  for (int i = 0; i<kMaxPrimaries; ++i){
    pdg[i] = -99999;
    Eng[i] = -99999;
    Px[i] = -99999;
    Py[i] = -99999;
    Pz[i] = -99999;
    StartPointx[i] = -99999;
    StartPointy[i] = -99999;
    StartPointz[i] = -99999;
    EndPointx[i] = -99999;
    EndPointy[i] = -99999;
    EndPointz[i] = -99999;
    NumberDaughters[i] = -99999;
    Mother[i] = -99999;
    TrackId[i] = -99999;
    process_primary[i] = -99999;
    orig[i] = -99999;
    genie_primaries_pdg[i] = -99999;
    genie_Eng[i] = -99999;
    genie_Px[i] = -99999;
    genie_Py[i] = -99999;
    genie_Pz[i] = -99999;
    genie_P[i] = -99999;
    genie_status_code[i] = -99999;
    genie_mass[i] = -99999;
    genie_trackID[i] = -99999;
    genie_ND[i] = -99999;
    genie_mother[i] = -99999;
  }    
  
  for (int i = 0; i<kMaxHits; ++i){
    hit_plane[i] = -99999;
    hit_wire[i] = -99999;
    hit_channel[i] = -99999;
    hit_peakT[i] = -99999;
    hit_peakTcorr[i] = -99999;
    hit_charge[i] = -99999;
    hit_ph[i] = -99999;
    hit_trkid[i] = -99999;
    hit_etruth[i] = -99999;
  }

//  for (int i = 0; i<2; ++i){
//    twodvtx_w_reco[i] = -99999;
//    twodvtx_t_reco[i] = -99999;
//    twodvtx_w_truth[i] = -99999;
//    twodvtx_t_truth[i] = -99999;
//  } 

//  nkingaclustersu_reco = -99999;
//  nkingaclustersv_reco = -99999;
//  nvertexkingaclustersu_reco = -99999;
//  nvertexkingaclustersv_reco = -99999;
//  
//  nlinemergerclustersu_reco = -99999;
//  nlinemergerclustersv_reco = -99999;
//  nvertexlinemergerclustersu_reco = -99999;
//  nvertexlinemergerclustersv_reco = -99999;
//  
//  ndbscanclustersu_reco = -99999;
//  ndbscanclustersv_reco = -99999;
//  nvertexdbscanclustersu_reco = -99999;
//  nvertexdbscanclustersv_reco = -99999;
  
//  no_kingaclusters = -99999;
//  no_linemergerclusters = -99999;   
//  no_fuzzyclusters = -99999;
  no_clusters = -99999;
  for (int i = 0; i<kMaxClusters; ++i){
//   kingaclusters_planeNo[i] = -99999;
//   Start_pt_w_kingaCl[i] = -99999;
//   Start_pt_t_kingaCl[i] = -99999;
//   linemergerclusters_planeNo[i] = -99999;
//   Start_pt_w_linemergerCl[i] = -99999;
//   Start_pt_t_linemergerCl[i] = -99999;
   fNhitsClust[i] = -99999;
   clusters_planeNo[i] = -99999;
   Start_pt_w_Cl[i] = -99999;
   Start_pt_t_Cl[i] = -99999;
   End_pt_w_Cl[i] = -99999;
   End_pt_t_Cl[i] = -99999;
  }
  
  for (int i = 0; i<2; ++i){
    two_trackstart_dcosx_reco[i] = -99999;
    two_trackstart_dcosy_reco[i] = -99999;
    two_trackstart_dcosz_reco[i] = -99999;
    two_trackexit_dcosx_reco[i] = -99999;
    two_trackexit_dcosy_reco[i] = -99999;
    two_trackexit_dcosz_reco[i] = -99999;
  }

  pmttime = -99999;
  for (int i = 0; i<4; ++i){
    pmt1[i] = -99999;
    pmt2[i] = -99999;
    pmt3[i] = -99999;
    pmt4[i] = -99999;
  }
}

bool t962::AnalysisTreeT962::EndsOnBoundary(art::Ptr<recob::Track> lar_track)
{
  std::vector<double> larStart, larEnd;
  lar_track->Extent(larStart,larEnd);//put xyz coordinates at begin/end of track into vectors(?)
  
  if(fabs(larEnd[0])<fboundaryWindow 
     || fabs(47.-larEnd[0])<fboundaryWindow 
     || fabs(larEnd[1]+20.)<fboundaryWindow
     || fabs(20.-larEnd[1])<fboundaryWindow 
     || fabs(larEnd[2])<fboundaryWindow 
     || fabs(90.-larEnd[2])<fboundaryWindow  )   
    return true;  
  else return false;
}




namespace t962{

  DEFINE_ART_MODULE(AnalysisTreeT962)
  
} 

#endif
